import os
import socket
import bsoncoder as bson
from struct import unpack
from select import select
from threading import Thread


class Processor(Thread):
    def __init__(self, client, callback):
        Thread.__init__(self)
        self.daemon = True
        self.client = client
        self.callback = callback
        self.project_id = None
        self.pid = None

    def drop(self):
        if self.project_id:
            self.callback.on_remove_project(self.project_id, self.pid)

        try:
            self.client.close()
        except:
            pass

    def run(self):
        while True:
            iready, oready, eready = select([self.client], [], [], 1)
            if self.client in iready:
                buf = self.client.recv(4)
                if len(buf) != 4:
                    return self.drop()
                length = unpack('<I', buf)[0]
                got = 4
                data = buf
                while got < length:
                    buf = self.client.recv(512 if length - got > 512 else length - got)
                    if len(buf) == 0:
                        return self.drop()
                    data += buf
                    got += len(buf)
                try:
                    record = bson.loads(data)
                except:
                    return self.drop()
                if record.get('action') == 'hello':
                    self.project_id = record.get('project_id')
                    self.pid = record.get('pid')
                    self.callback.on_new_project(self.project_id, self.pid)
                elif record.get('action') == 'log_event':
                    if not self.project_id:
                        return self.drop()
                    self.callback.on_log_event(self.project_id, record.get('record'))

class Server(Thread):
    def __init__(self, callback):
        Thread.__init__(self)
        self.server = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        try:
            self.server.connect('/tmp/overwatch.sock')
            raise Exception('Another instance of overwatch browser is already running.')
        except socket.error:
            try:
                os.remove('/tmp/overwatch.sock')
            except OSError:
                pass
            pass

        self.server.bind('/tmp/overwatch.sock')
        self.server.listen(10)
        self.callback = callback
        self.daemon = True

    def run(self):
        while True:
            client, info = self.server.accept()
            watcher = Processor(client, self.callback)
            watcher.start()



from threading import Thread
from Queue import Queue
import logging
import socket
import bsoncoder as bson
import os
import datetime
import traceback
from time import time, sleep


def init(id, api_key=None):
    _instance = Overwatch(id, api_key)
    _instance.start()

class Overwatch(Thread):
    class Handler(logging.StreamHandler):
        def __init__(self, callback):
            self.callback = callback
            logging.StreamHandler.__init__(self)

        def emit(self, record):
            self.callback(record)

    def __init__(self, id, api_key):
        Thread.__init__(self)
        self.daemon = True
        self.id = id
        self.api_key = api_key
        self.socket_path = '/tmp/overwatch-%s.sock' % self.id

        self._init_socket()

        self.queue = Queue()
        self.root_logger = logging.getLogger()
        self.root_logger.addHandler(Overwatch.Handler(self._process))

    def _init_socket(self):
        self.connect()

    def connect(self):
        try:
            self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            self.socket.connect('/tmp/overwatch.sock')
            self.socket.send(bson.dumps({
                'action': 'hello',
                'project_id': self.id,
                'pid': os.getpid(),
            }))
        except Exception, e:
            pass

    def _process(self, record):
        self.queue.put(record)

    def run(self):
        while True:
            try:
                record = self.queue.get(True, 1)
            except:
                continue
            if record.exc_info:
                record.msg = record.msg + '\n' + ''.join(traceback.format_exception(
                    record.exc_info[0],
                    record.exc_info[1],
                    record.exc_info[2],
                    100
                ))
            while True:
                try:
                    self.socket.send(bson.dumps({
                        'action': 'log_event',
                        'record': {
                            'source': 'python',
                            'levelname': record.levelname,
                            'levelno': record.levelno,
                            'api_key': self.api_key,
                            'module': record.module,
                            'filename': record.filename,
                            'func_name': record.funcName,
                            'lineno': str(record.lineno),
                            'msg': record.msg % record.args,
                            'time': '%f' % time(),
                        },
                    }))
                    break
                except Exception, e:
                    sleep(1)
                    self.connect()



#!/usr/bin/python -OOOO
# vim: set fileencoding=utf8 shiftwidth=4 tabstop=4 textwidth=80 foldmethod=marker :
# Copyright (c) 2010, Kou Man Tong. All rights reserved.
# For licensing, see LICENSE file included in the package.
"""
Base codec functions for bson.
"""
import struct
import cStringIO
import calendar, pytz
from datetime import datetime
import warnings
from abc import ABCMeta, abstractmethod

# {{{ Error Classes
class MissingClassDefinition(ValueError):
	def __init__(self, class_name):
		super(MissingClassDefinition, self).__init__(
		"No class definition for class %s" % (class_name,))
# }}}
# {{{ Warning Classes
class MissingTimezoneWarning(RuntimeWarning):
	def __init__(self, *args):
		args = list(args)
		if len(args) < 1:
			args.append("Input datetime object has no tzinfo, assuming UTC.")
		super(MissingTimezoneWarning, self).__init__(*args)
# }}}
# {{{ Traversal Step
class TraversalStep(object):
	def __init__(self, parent, key):
		self.parent = parent
		self.key = key
# }}}
# {{{ Custom Object Codec

class BSONCoding(object):
	__metaclass__ = ABCMeta

	@abstractmethod
	def bson_encode(self):
		pass

	@abstractmethod
	def bson_init(self, raw_values):
		pass

classes = {}

def import_class(cls):
	if not issubclass(cls, BSONCoding):
		return

	global classes
	classes[cls.__name__] = cls

def import_classes(*args):
	for cls in args:
		import_class(cls)

def import_classes_from_modules(*args):
	for module in args:
		for item in module.__dict__:
			if hasattr(item, "__new__") and hasattr(item, "__name__"):
				import_class(item)

def encode_object(obj, traversal_stack, generator_func):
	values = obj.bson_encode()
	class_name = obj.__class__.__name__
	values["$$__CLASS_NAME__$$"] = class_name
	return encode_document(values, traversal_stack, obj, generator_func)

def encode_object_element(name, value, traversal_stack, generator_func):
	return "\x03" + encode_cstring(name) + \
			encode_object(value, traversal_stack,
					generator_func = generator_func)

class _EmptyClass(object):
	pass

def decode_object(raw_values):
	global classes
	class_name = raw_values["$$__CLASS_NAME__$$"]
	cls = None
	try:
		cls = classes[class_name]
	except KeyError, e:
		raise MissingClassDefinition(class_name)

	retval = _EmptyClass()
	retval.__class__ = cls
	alt_retval = retval.bson_init(raw_values)
	return alt_retval or retval

# }}}
# {{{ Codec Logic
def encode_string(value):
	value = value.encode("utf8")
	length = len(value)
	return struct.pack("<i%dsb" % (length,), length + 1, value, 0)

def decode_string(data, base):
	length = struct.unpack("<i", data[base:base + 4])[0]
	value = data[base + 4: base + 4 + length - 1]
	value = value.decode("utf8")
	return (base + 4 + length, value)

def encode_cstring(value):
	if isinstance(value, unicode):
		value = value.encode("utf8")
	return value + "\x00"

def decode_cstring(data, base):
	length = 0
	max_length = len(data) - base
	while length < max_length:
		character = data[base + length]
		length += 1
		if character == "\x00":
			break
	return (base + length, data[base:base + length - 1].decode("utf8"))

def encode_binary(value):
	length = len(value)
	return struct.pack("<ib", length, 0) + value

def decode_binary(data, base):
	length, binary_type = struct.unpack("<ib", data[base:base + 5])
	return (base + 5 + length, data[base + 5:base + 5 + length])

def encode_double(value):
	return struct.pack("<d", value)

def decode_double(data, base):
	return (base + 8, struct.unpack("<d", data[base: base + 8])[0])


ELEMENT_TYPES = {
		0x01 : "double",
		0x02 : "string",
		0x03 : "document",
		0x04 : "array",
		0x05 : "binary",
		0x08 : "boolean",
        0x09 : "UTCdatetime",
		0x0A : "none",
		0x10 : "int32",
		0x12 : "int64"
	}

def encode_double_element(name, value):
	return "\x01" + encode_cstring(name) + encode_double(value)

def decode_double_element(data, base):
	base, name = decode_cstring(data, base + 1)
	base, value = decode_double(data, base)
	return (base, name, value)

def encode_string_element(name, value):
	return "\x02" + encode_cstring(name) + encode_string(value)

def decode_string_element(data, base):
	base, name = decode_cstring(data, base + 1)
	base, value = decode_string(data, base)
	return (base, name, value)

def encode_value(name, value, buf, traversal_stack, generator_func):
	if isinstance(value, BSONCoding):
		buf.write(encode_object_element(name, value, traversal_stack,
			generator_func))
	elif isinstance(value, float):
		buf.write(encode_double_element(name, value))
	elif isinstance(value, unicode):
		buf.write(encode_string_element(name, value))
	elif isinstance(value, dict):
		buf.write(encode_document_element(name, value,
			traversal_stack, generator_func))
	elif isinstance(value, list) or isinstance(value, tuple):
		buf.write(encode_array_element(name, value,
			traversal_stack, generator_func))
	elif isinstance(value, str):
		buf.write(encode_binary_element(name, value))
	elif isinstance(value, bool):
		buf.write(encode_boolean_element(name, value))
	elif isinstance(value, datetime):
		buf.write(encode_UTCdatetime_element(name, value))
	elif value is None:
		buf.write(encode_none_element(name, value))
	elif isinstance(value, int):
		if value < -0x80000000 or value > 0x7fffffff:
			buf.write(encode_int64_element(name, value))
		else:
			buf.write(encode_int32_element(name, value))
	elif isinstance(value, long):
		buf.write(encode_int64_element(name, value))

def encode_document(obj, traversal_stack,
		traversal_parent = None,
		generator_func = None):
	buf = cStringIO.StringIO()
	key_iter = obj.iterkeys()
	if generator_func is not None:
		key_iter = generator_func(obj, traversal_stack)
	for name in key_iter:
		value = obj[name]
		traversal_stack.append(TraversalStep(traversal_parent or obj, name))
		encode_value(name, value, buf, traversal_stack, generator_func)
		traversal_stack.pop()
	e_list = buf.getvalue()
	e_list_length = len(e_list)
	return struct.pack("<i%dsb" % (e_list_length,), e_list_length + 4 + 1,
			e_list, 0)

def encode_array(array, traversal_stack,
		traversal_parent = None,
		generator_func = None):
	buf = cStringIO.StringIO()
	for i in xrange(0, len(array)):
		value = array[i]
		traversal_stack.append(TraversalStep(traversal_parent or array, i))
		encode_value(unicode(i), value, buf, traversal_stack, generator_func)
		traversal_stack.pop()
	e_list = buf.getvalue()
	e_list_length = len(e_list)
	return struct.pack("<i%dsb" % (e_list_length,), e_list_length + 4 + 1,
			e_list, 0)

def decode_element(data, base):
	element_type = struct.unpack("<b", data[base:base + 1])[0]
	element_description = ELEMENT_TYPES[element_type]
	decode_func = globals()["decode_" + element_description + "_element"]
	return decode_func(data, base)

def decode_document(data, base):
	length = struct.unpack("<i", data[base:base + 4])[0]
	end_point = base + length
	base += 4
	retval = {}
	while base < end_point - 1:
		base, name, value = decode_element(data, base)
		retval[name] = value
	if "$$__CLASS_NAME__$$" in retval:
		retval = decode_object(retval)
	return (end_point, retval)

def encode_document_element(name, value, traversal_stack, generator_func):
	return "\x03" + encode_cstring(name) + \
			encode_document(value, traversal_stack,
					generator_func = generator_func)

def decode_document_element(data, base):
	base, name = decode_cstring(data, base + 1)
	base, value = decode_document(data, base)
	return (base, name, value)

def encode_array_element(name, value, traversal_stack, generator_func):
	return "\x04" + encode_cstring(name) + \
			encode_array(value, traversal_stack, generator_func = generator_func)

def decode_array_element(data, base):
	base, name = decode_cstring(data, base + 1)
	base, value = decode_document(data, base)
	retval = []
	try:
		i = 0
		while True:
			retval.append(value[unicode(i)])
			i += 1
	except KeyError:
		pass
	return (base, name, retval)

def encode_binary_element(name, value):
	return "\x05" + encode_cstring(name) + encode_binary(value)

def decode_binary_element(data, base):
	base, name = decode_cstring(data, base + 1)
	base, value = decode_binary(data, base)
	return (base, name, value)

def encode_boolean_element(name, value):
	return "\x08" + encode_cstring(name) + struct.pack("<b", value)

def decode_boolean_element(data, base):
	base, name = decode_cstring(data, base + 1)
	value = not not struct.unpack("<b", data[base:base + 1])[0]
	return (base + 1, name, value)

def encode_UTCdatetime_element(name, value):
	if value.tzinfo is None:
		warnings.warn(MissingTimezoneWarning(), None, 4)
	value = int(round(calendar.timegm(value.utctimetuple()) * 1000 +
		(value.microsecond / 1000.0)))
	return "\x09" + encode_cstring(name) + struct.pack("<q", value)

def decode_UTCdatetime_element(data, base):
	base, name = decode_cstring(data, base + 1)
	value = datetime.fromtimestamp(struct.unpack("<q",
		data[base:base + 8])[0] / 1000.0, pytz.utc)
	return (base + 8, name, value)

def encode_none_element(name, value):
	return "\x0a" + encode_cstring(name)

def decode_none_element(data, base):
	base, name = decode_cstring(data, base + 1)
	return (base, name, None)

def encode_int32_element(name, value):
	return "\x10" + encode_cstring(name) + struct.pack("<i", value)

def decode_int32_element(data, base):
	base, name = decode_cstring(data, base + 1)
	value = struct.unpack("<i", data[base:base + 4])[0]
	return (base + 4, name, value)

def encode_int64_element(name, value):
	return "\x12" + encode_cstring(name) + struct.pack("<q", value)

def decode_int64_element(data, base):
	base, name = decode_cstring(data, base + 1)
	value = struct.unpack("<q", data[base:base + 8])[0]
	return (base + 8, name, value)
# }}}



#!/usr/bin/python -OOOO
# vim: set fileencoding=utf8 shiftwidth=4 tabstop=4 textwidth=80 foldmethod=marker :
# Copyright (c) 2010, Kou Man Tong. All rights reserved.
# For licensing, see LICENSE file included in the package.
"""
BSON serialization and deserialization logic.
Specifications taken from: http://bsonspec.org/#/specification
The following types are unsupported, because for data exchange purposes, they're
over-engineered:
	0x06 (Undefined)
	0x07 (ObjectId)
	0x0b (Regex - Exactly which flavor do you want? Better let higher level
		programmers make that decision.)
	0x0c (DBPointer)
	0x0d (JavaScript code)
	0x0e (Symbol)
	0x0f (JS w/ scope)
	0x11 (MongoDB-specific timestamp)

For binaries, only the default 0x0 type is supported.


>>> a = {
...   u"Item A" : u"String item A",
...   u"Item D" : {u"ROFLOL" : u"Blah blah blah"},
...   u"Item C" : [1, 123456789012345, None, "Party and Bad Romance"],
...   u"Item B" : u"\u4e00\u9580\u4e94\u5091"
... }
>>> def sorted(obj, dfs_stack):
...   keys = obj.keys()
...   keys.sort()
...   for i in keys: yield i
... 
>>> def reverse(obj, dfs_stack):
...   keys = obj.keys()
...   keys.sort(reverse = True)
...   for i in keys: yield i
... 
>>> serialized = dumps(a, sorted)
>>> serialized
'\\x9f\\x00\\x00\\x00\\x02Item A\\x00\\x0e\\x00\\x00\\x00String item A\\x00\\x02Item B\\x00\\r\\x00\\x00\\x00\\xe4\\xb8\\x80\\xe9\\x96\\x80\\xe4\\xba\\x94\\xe5\\x82\\x91\\x00\\x04Item C\\x007\\x00\\x00\\x00\\x100\\x00\\x01\\x00\\x00\\x00\\x121\\x00y\\xdf\\r\\x86Hp\\x00\\x00\\n2\\x00\\x053\\x00\\x15\\x00\\x00\\x00\\x00Party and Bad Romance\\x00\\x03Item D\\x00 \\x00\\x00\\x00\\x02ROFLOL\\x00\\x0f\\x00\\x00\\x00Blah blah blah\\x00\\x00\\x00'
>>> 
>>> b = loads(serialized)
>>> b
{u'Item C': [1, 123456789012345, None, 'Party and Bad Romance'], u'Item B': u'\\u4e00\\u9580\\u4e94\\u5091', u'Item A': u'String item A', u'Item D': {u'ROFLOL': u'Blah blah blah'}}
>>> reverse_serialized = dumps(a, reverse)
>>> reverse_serialized
'\\x9f\\x00\\x00\\x00\\x03Item D\\x00 \\x00\\x00\\x00\\x02ROFLOL\\x00\\x0f\\x00\\x00\\x00Blah blah blah\\x00\\x00\\x04Item C\\x007\\x00\\x00\\x00\\x100\\x00\\x01\\x00\\x00\\x00\\x121\\x00y\\xdf\\r\\x86Hp\\x00\\x00\\n2\\x00\\x053\\x00\\x15\\x00\\x00\\x00\\x00Party and Bad Romance\\x00\\x02Item B\\x00\\r\\x00\\x00\\x00\\xe4\\xb8\\x80\\xe9\\x96\\x80\\xe4\\xba\\x94\\xe5\\x82\\x91\\x00\\x02Item A\\x00\\x0e\\x00\\x00\\x00String item A\\x00\\x00'
>>> c = loads(reverse_serialized)
>>> c
{u'Item C': [1, 123456789012345, None, 'Party and Bad Romance'], u'Item B': u'\\u4e00\\u9580\\u4e94\\u5091', u'Item A': u'String item A', u'Item D': {u'ROFLOL': u'Blah blah blah'}}
"""

from codec import *
import network
__all__ = ["loads", "dumps"]

# {{{ Serialization and Deserialization
def dumps(obj, generator = None):
	"""
	Given a dict, outputs a BSON string.

	generator is an optional function which accepts the dictionary/array being
	encoded, the current DFS traversal stack, and outputs an iterator indicating
	the correct encoding order for keys.
	"""
	if isinstance(obj, BSONCoding):
		return encode_object(obj, [], generator_func = generator)
	return encode_document(obj, [], generator_func = generator)

def loads(data):
	"""
	Given a BSON string, outputs a dict.
	"""
	return decode_document(data, 0)[1]
# }}}
# {{{ Socket Patchers
def patch_socket():
	"""
	Patches the Python socket class such that sockets can send and receive BSON
	objects atomically.

	This adds the following functions to socket:

	recvbytes(bytes_needed, sock_buf = None) - reads bytes_needed bytes
	atomically. Returns None if socket closed.

	recvobj() - reads a BSON document from the socket atomically and returns
	the deserialized dictionary. Returns None if socket closed.

	sendobj(obj) - sends a BSON document to the socket atomically. 
	"""
	from socket import socket
	socket.recvbytes = network._recvbytes
	socket.recvobj = network._recvobj
	socket.sendobj = network._sendobj
# }}}



import os
import pkg_resources
import sys
import logging
from threading import Thread
import time
import re
import math
import datetime
from server import Server

log = logging.getLogger('root')

#DIR = os.path.dirname(__file__)


get_resource_path = lambda filename: pkg_resources.resource_filename('overwatch', os.path.join('data', filename))

try:
    import pygtk
    pygtk.require('2.0')
except:
    pass
try:
    import gtk
    import gtk.glade
    import gobject
    import pango
    gobject.threads_init()
except Exception, e:
    log.error('Could not load GTK libs, error was: %s' % e)
    raise e
    sys.exit(1)

'''gtk.window_set_default_icon_list(
    gtk.gdk.pixbuf_new_from_file('images/icon256.png'),
    gtk.gdk.pixbuf_new_from_file('images/icon128.png'),
    gtk.gdk.pixbuf_new_from_file('images/icon64.png'),
    gtk.gdk.pixbuf_new_from_file('images/icon48.png'),
    gtk.gdk.pixbuf_new_from_file('images/icon32.png'),
    gtk.gdk.pixbuf_new_from_file('images/icon16.png'),
)'''


def defer(method, *args):
    gobject.idle_add(method, *args)


def deferred(fn):
    def wrapper(self, *args, **kwargs):
        defer(fn, *[self] + list(args))
    return wrapper


class Form:
    '''
    Generic form class.
    Loads a form ant initializes main widget.
    '''
    def __init__(self, gui, form_id):
        ''' Load form, connect signals. '''
        self.glade = gtk.Builder()
        f = open(get_resource_path('%s.glade' % (form_id)), 'r')
        xml = f.read().replace('GtkBox" id="h', 'GtkHBox" id="h').replace('GtkBox" id="v', 'GtkVBox" id="v')
        xml = '\n'.join([line for line in xml.split('\n') if line.find('requires') < 0])
        f.close()
        self.glade.add_from_string(xml)
        self.window = self.glade.get_object(form_id)
        self.gui = gui
        self.glade.connect_signals(self)

    def show(self):
        ''' Show form. '''
        self.window.show()

    def __getattr__(self, name):
        ''' Delegates unexisting properties to parent GUI object. '''
        if name.startswith('id_'):
            widget_matches = [x for x in self.glade.get_objects() if isinstance(x, (gtk.Buildable, gtk.ListStore)) and gtk.Buildable.get_name(x) == name[3:]]
            if len(widget_matches):
                return widget_matches[0]
        elif name.endswith('_cb'):
            def delegate_event(*args):
                self.gui.on_event(name, args)
            return delegate_event
        else:
            raise AttributeError('Attribute \'%s\' not found in Form (or its derivatives).' % name)


class MainForm(Form):
    '''
    Main form.
    '''

    def __init__(self, gui):
        ''' Inits main window with map, skills, chat etc. '''
        # Call parent constructor in order to properly create form.
        Form.__init__(self, gui, 'form')
        self.window.maximize()
        self.window.connect('destroy', gtk.main_quit)

        self.log_items_filter = self.id_log_items_store.filter_new()
        self.log_items_filter.set_visible_column(14)
        self.id_log_items_tree.set_model(self.log_items_filter)

        self.id_log_items_filter_entry.connect('changed', self.on_filter_change)

    def on_filter_change(self, entry):
        store = self.id_log_items_store
        text = entry.get_text().lower()
        for row in store:
            matched = False
            for i in (1, 5, 9):
                if row[i].lower().find(text) != -1:
                    matched = True
            row[14] = matched

class GUI:
    LEVEL_COLORS = {
        'DEBUG': (gtk.gdk.color_parse('#AAFFFF'), gtk.gdk.color_parse('#007777')),
        'INFO': (gtk.gdk.color_parse('#00AA00'), gtk.gdk.color_parse('#FFFFFF')),
        'WARNING': (gtk.gdk.color_parse('#AAAA00'), gtk.gdk.color_parse('#FFFFFF')),
        'ERROR': (gtk.gdk.color_parse('#AA0000'), gtk.gdk.color_parse('#FFFFFF')),
        'EXCEPTION': (gtk.gdk.color_parse('#AA0000'), gtk.gdk.color_parse('#FFFFFF')),
    }

    CRC_COLORS = {
        0: gtk.gdk.color_parse('#AA7777'),
        1: gtk.gdk.color_parse('#77AA77'),
        2: gtk.gdk.color_parse('#7777AA'),
        3: gtk.gdk.color_parse('#AA0077'),
        4: gtk.gdk.color_parse('#AA7700'),
        5: gtk.gdk.color_parse('#00AA77'),
        6: gtk.gdk.color_parse('#77AA00'),
        7: gtk.gdk.color_parse('#0077AA'),
        8: gtk.gdk.color_parse('#7700AA'),
        9: gtk.gdk.color_parse('#AA0000'),
        10: gtk.gdk.color_parse('#00AA00'),
        11: gtk.gdk.color_parse('#0000AA'),
        12: gtk.gdk.color_parse('#00AAAA'),
        13: gtk.gdk.color_parse('#AA00AA'),
        14: gtk.gdk.color_parse('#AAAA00'),
        15: gtk.gdk.color_parse('#77AAAA'),
        16: gtk.gdk.color_parse('#AA77AA'),
    }

    def __init__(self):
        self.main_form = MainForm(self)
        self.main_form.show()

        self.i = 0

        self.server = Server(self)
        self.server.start()

    def crc(self, s):
        return sum([ord(x) for x in s]) % len(GUI.CRC_COLORS)

    @deferred
    def on_log_event(self, project_id, record):
        self.i += 1
        msg_formatted = record.get('msg').replace('<', '&lt;').replace('>', '&gt;')
        msg_formatted = re.sub('File (.*), line ([\d]+), in (.*)', 'File <span foreground="#0055AA">\\1</span>, line <span foreground="#AA0000">\\2</span>, in <span foreground="#AA0000">\\1</span>', msg_formatted)
        msg_formatted = re.sub('\&lt\;(.*)\&gt\;', '<span foreground="#FF1177" weight="bold">&lt;\\1&gt;</span>', msg_formatted)
        msg_formatted = re.sub('\{([^\{]*)\}', '<span foreground="#00AAFF" weight="bold">{\\1}</span>', msg_formatted)
        self.main_form.id_log_items_store.append([
            self.i,
            #'<span foreground="#FFFFFF" background="#00AA00">%s</span>' % project_id,
            project_id,
            record.get('levelname'),
            int(record.get('levelno')),
            record.get('source'),
            record.get('module'),
            record.get('filename'),
            record.get('func_name'),
            int(record.get('lineno')),
            msg_formatted,
            datetime.datetime.fromtimestamp(float(record.get('time'))).strftime('%H:%M:%S'),
            GUI.LEVEL_COLORS[record.get('levelname')][0],
            GUI.LEVEL_COLORS[record.get('levelname')][1],
            GUI.CRC_COLORS[self.crc(project_id)],
            True,
        ])

        tree = self.main_form.id_log_items_tree
        tree.set_cursor_on_cell((len(self.main_form.log_items_filter),))

    @deferred
    def on_new_project(self, project_id, pid):
        self.main_form.id_projects_store.append([project_id, pid, GUI.CRC_COLORS[self.crc(project_id)]])

    @deferred
    def on_remove_project(self, project_id, pid):
        store = self.main_form.id_projects_store
        for row in store:
            if row[1] == pid:
                store.remove(row.iter)

    def main(self):
        gtk.main()



#!/usr/bin/env python2

'''Overwatch: Python Logging Browser.

This package includes special logger and GUI written in GTK2
that allows you to perform real-time logs monitoring from a
bunch of applications.
'''

from distutils.core import setup
from setuptools import find_packages

classifiers = '''
Development Status :: 4 - Beta
Environment :: Console
Environment :: X11 Applications :: GTK
Intended Audience :: Developers
Intended Audience :: End Users/Desktop
Intended Audience :: System Administrators
License :: OSI Approved :: GNU General Public License v2 (GPLv2)
Operating System :: POSIX :: Linux
Programming Language :: Python :: 2.7
Topic :: Software Development :: Debuggers
Topic :: System :: Monitoring
'''

doclines = __doc__.split('\n')

setup(name='overwatch',
        version='1.1',
        author='Andrew Dunai',
        author_email='andunai@gmail.com',
        url='http://andrewdunai.com/projects/overwatch/',
        packages=find_packages(),
        package_data={'overwatch': ['data/*']},
        platforms=['unix'],
        classifiers=filter(None, classifiers.split("\n")),
        description=doclines[0],
        long_description='\n'.join(doclines[2:]),
     )



#!/usr/bin/env python

import socket
try:
	from cStringIO import StringIO
except ImportError, e:
	from StringIO import StringIO
from struct import unpack
from __init__ import dumps, loads

def _bintoint(data):
	return unpack("<i", data)[0]

def _sendobj(self, obj):
	"""
	Atomically send a BSON message.
	"""
	data = dumps(obj)
	self.sendall(data)

def _recvobj(self):
	"""
	Atomic read of a BSON message.

	This function either returns a dict, None, or raises a socket error.

	If the return value is None, it means the socket is closed by the other side.
	"""
	sock_buf = self.recvbytes(4)
	if sock_buf is None:
		return None

	message_length = _bintoint(sock_buf.getvalue())
	sock_buf = self.recvbytes(message_length - 4, sock_buf)
	if sock_buf is None:
		return None

	retval = loads(sock_buf.getvalue())
	return retval


def _recvbytes(self, bytes_needed, sock_buf = None):
	"""
	Atomic read of bytes_needed bytes.

	This function either returns exactly the nmber of bytes requested in a
	StringIO buffer, None, or raises a socket error.

	If the return value is None, it means the socket is closed by the other side.
	"""
	if sock_buf is None:
		sock_buf = StringIO()
	bytes_count = 0
	while bytes_count < bytes_needed:
		chunk = self.recv(min(bytes_needed - bytes_count, 32768))
		part_count = len(chunk)

		if part_count < 1:
			return None

		bytes_count += part_count
		sock_buf.write(chunk)
	
	return sock_buf
